#ifndef __CBITBOARD__
#define __CBITBOARD__

#include "config.h"

#include "types.h"

inline unsigned int Low(BITBOARD b){
	return (unsigned int)(b & 0xFFFFFFFF);
}
inline unsigned int Hi(BITBOARD b){
	return (unsigned int)(b >> 32);
}

void Print(BITBOARD b, int level = 0);
void PrintBitboard(BITBOARD b);

// bitboard operations
#define Compl(b) (~(b))
#define Xor(b1, b2) ((b1) ^ (b2))
#define And(b1, b2) ((b1) & (b2))
#define Or(b1, b2)	((b1) | (b2))
#define Shl(b, i)	((b) << (i))
#define Shr(b, i)	((b) >> (i))
#define OneShl(i)	Shl((BITBOARD)1, i)
#define Not(i) (!(i))


#define IsSet(b, bit)	((b) & OneShl(bit))
#define Set(b, bit)		b |= (((BITBOARD)1) << (bit));
#define Clear(b, bit)	b &= ~(((BITBOARD)1) << (bit));

#define ClearFirstOne(x) (x) = ((x) & ((x)-1));

BITBOARD Mirror(BITBOARD& b);

void MirrorAndSwap(BITBOARD& b1, BITBOARD& b2);


int FirstOne(BITBOARD b);
int LastOne(BITBOARD b);

#if VISUAL_C

// Visual C++ specific stuff, from Crafty
#if _MSC_VER >= 1200
#define FORCEINLINE __forceinline
#else
#define FORCEINLINE __inline
#endif

#pragma warning ( disable: 4035 )


FORCEINLINE int BitCount(BITBOARD a) {
  __asm {
        mov     ecx, dword ptr a
        xor     eax, eax
        test    ecx, ecx
        jz      l1
    l0: lea     edx, [ecx-1]
        inc     eax
        and     ecx, edx
        jnz     l0
    l1: mov     ecx, dword ptr a+4
        test    ecx, ecx
        jz      l3
    l2: lea     edx, [ecx-1]
        inc     eax
        and     ecx, edx
        jnz     l2
    l3: 
  }
}


FORCEINLINE int FirstOne(BITBOARD a) {
  __asm {
        xor eax, eax
		bsf     eax, dword ptr a
		jnz		l1
		bsf		eax, dword ptr a+4
		jz		l1
		add		eax, 32
	l1:
  }
}

FORCEINLINE int LastOne(BITBOARD a) {
  __asm {	
        xor		edx, edx
        bsr     edx, dword ptr a+4
		mov		eax, 32
		jnz		l1
		xor		eax, eax
		bsr		edx, dword ptr a
	l1:
		add		eax, edx
  }
}

#pragma warning ( default: 4035 )


#else 

int BitCount(BITBOARD a);

int FirstOne(BITBOARD a);
int LastOne(BITBOARD a);

#endif


#endif