#include "board.h"
#include "move.h"

#include "tree.h"

#include "hash.h"

#include "history.h"

#include "prophet.h"

#include <string.h>
#include <iostream>

using namespace std;

#define abs(x) (((x)>0)?(x):(-(x)))

Board board;

SaveInfo save_struct[MAX_PLY];

#define VERIFY_BOARD 0

BITBOARD AttacksFrom(int square, int wtm){
	switch(abs(PieceOnSquare(square))){
	case PAWN:
		if (wtm) return wpawn_attacks[square];
		else return bpawn_attacks[square];
	case KNIGHT:
		return knight_attacks[square];
	case BISHOP:
		return AttacksBishop(square);
	case ROOK:
		return AttacksRook(square);
	case QUEEN:
		return AttacksQueen(square);
	case KING:
		return king_attacks[square];
	default:
		return 0;
	}
}

BITBOARD AttacksTo(int square, int wtm){
	return Or(Or(Or(Or(Or(And(wpawn_attacks[square], board.bPawns),
						  And(bpawn_attacks[square], board.wPawns)),
					   And(knight_attacks[square], Or(board.bKnights,board.wKnights))),
					   And(AttacksBishop(square), board.bishops_queens)),
					   And(AttacksRook(square), board.rooks_queens)),
					   And(king_attacks[square], Or(board.bKing,board.wKing)));
}

// checks if wtm attacks a square, same as AttacksTo, but it doesn't return a bitboard 
int Attacked(int square, int wtm){
	if (wtm){
		if (And(bpawn_attacks[square], board.wPawns)) return 1;
		if (And(knight_attacks[square], board.wKnights)) return 1;
		if (And(And(AttacksBishop(square),board.bishops_queens),board.white_pieces)) return 1;
		if (And(And(AttacksRook(square),board.rooks_queens),board.white_pieces)) return 1;
		if (And(king_attacks[square], board.wKing)) return 1;
		return 0;
	} else {
		if (And(wpawn_attacks[square], board.bPawns)) return 1;
		if (And(knight_attacks[square], board.bKnights)) return 1;
		if (And(And(AttacksBishop(square),board.bishops_queens),board.black_pieces)) return 1;
		if (And(And(AttacksRook(square),board.rooks_queens),board.black_pieces)) return 1;
		if (And(king_attacks[square], board.bKing)) return 1;
		return 0;
	}
}


/** Index for board rotation of 90 degrees left */

static const unsigned char index90[] = {
	7,15,23,31,39,47,55,63,
	6,14,22,30,38,46,54,62,
	5,13,21,29,37,45,53,61,
	4,12,20,28,36,44,52,60,
	3,11,19,27,35,43,51,59,
	2,10,18,26,34,42,50,58,
	1, 9,17,25,33,41,49,57,
	0, 8,16,24,32,40,48,56,
};

  /** Index for board rotation of 45 degrees left */
static const unsigned char index_a1[] = {
	28,36,43,49,54,58,61,63,
	21,29,37,44,50,55,59,62,
	15,22,30,38,45,51,56,60,
	10,16,23,31,39,46,52,57,
	 6,11,17,24,32,40,47,53,
	 3, 7,12,18,25,33,41,48,
	 1, 4, 8,13,19,26,34,42,
	 0, 2, 5, 9,14,20,27,35
};
		 
/** Index for board rotation of 45 degrees right*/
static const unsigned char index_h1[] = {
	63,62,60,57,53,48,42,35,
	61,59,56,52,47,41,34,27,
	58,55,51,46,40,33,26,20,
	54,50,45,39,32,25,19,14,
	49,44,38,31,24,18,13, 9,
	43,37,30,23,17,12, 8, 5,
	36,29,22,16,11, 7, 4, 2,
	28,21,15,10, 6, 3, 1, 0};

BITBOARD InitBitboard(const BITBOARD& b,const unsigned char index[]) {
	BITBOARD result = 0;
	
	for (int i = 0; i < 64; i++)
		if (IsSet(b,i))
  			Set(result,index[i]);
  	  
	return result;
}

unsigned char index[] = {
	56,57,58,59,60,61,62,63,
	48,49,50,51,52,53,54,55,
	40,41,42,43,44,45,46,47,
	32,33,34,35,36,37,38,39,
	24,25,26,27,28,29,30,31,
	16,17,18,19,20,21,22,23,
	8, 9,10,11,12,13,14,15,
	0, 1, 2, 3, 4, 5, 6, 7
};


extern int MAX_POSN_SCORE;


void Board::Init(char* epd){
	age = 0;
	
	MAX_POSN_SCORE = 2000;
	
	ClearHT();
	ClearPawnHT();

	ResetHistory();

	memset(sq, 0, sizeof(sq));

	int i,k;

	wPawns = wRooks = wKnights = wBishops = wQueens = wKing = 0;
	bPawns = bRooks = bKnights = bBishops = bQueens = bKing = 0;
	ep_square = 0;
	castle_flags = 0;	

	for (i = k = 0; k < (signed)strlen(epd) && (i < 64) && (epd[k] != ' '); k++) {
		BITBOARD mask = OneShl(index[i]);
    
		switch (epd[k]) {
			case 'K' : wKing   |= mask;sq[index[i]]=KING;i++;break;
			case 'P' : wPawns  |= mask;sq[index[i]]=PAWN;i++;break;
			case 'R' : wRooks  |= mask;sq[index[i]]=ROOK;i++;break;
			case 'N' : wKnights|= mask;sq[index[i]]=KNIGHT;i++;break;
			case 'B' : wBishops|= mask;sq[index[i]]=BISHOP;i++;break;
			case 'Q' : wQueens |= mask;sq[index[i]]=QUEEN;i++;break;
			case 'k' : bKing   |= mask;sq[index[i]]=-KING;i++;break;
			case 'p' : bPawns  |= mask;sq[index[i]]=-PAWN;i++;break;
			case 'r' : bRooks  |= mask;sq[index[i]]=-ROOK;i++;break;
			case 'n' : bKnights|= mask;sq[index[i]]=-KNIGHT;i++;break;
			case 'b' : bBishops|= mask;sq[index[i]]=-BISHOP;i++;break;
			case 'q' : bQueens |= mask;sq[index[i]]=-QUEEN;i++;break;
			case '8' : i+=8;break;
			case '7' : i+=7;break;
			case '6' : i+=6;break;
			case '5' : i+=5;break;
			case '4' : i+=4;break;
			case '3' : i+=3;break;
			case '2' : i+=2;break;
			case '1' :  i++;break;
			case '/' : if ((i % 8) != 0) cout << "Error in epd decoding " << i << endl;
		}
	}
  
	if (i != 64) cout << "Error in epd decoding " << i << endl;
    
	if (k < (signed)strlen(epd) && (epd[k] == ' ') && (epd[k+1] == 'b'))
		side_to_move = BLACK;
	else
		side_to_move = WHITE;

	// read castle rights
	k++;k++;
	while (epd[k] && (epd[k] == '	' || epd[k] == ' '))
		k++;

	board.castle_flags = 0;
	if (epd[k] == 'K') { board.castle_flags |= WHITE_SCASTLE; k++; }
	if (epd[k] == 'Q') { board.castle_flags |= WHITE_LCASTLE; k++; }
	if (epd[k] == 'k') { board.castle_flags |= BLACK_SCASTLE; k++; }
	if (epd[k] == 'q') { board.castle_flags |= BLACK_LCASTLE; k++; }
	k++;

	// read en passant square
	while (epd[k] && (epd[k] == '	' || epd[k] == ' '))
		k++;
	if (epd[k+1] == '6' && side_to_move == WHITE){
		int file = epd[k]-'a';
		if (file >= 0 && file <=7)
			board.ep_square = A6 + file;
	} else if (epd[k+1] == '3' && side_to_move == BLACK){
		int file = epd[k]-'a';
		if (file >= 0 && file <=7)
			board.ep_square = A3 + file;
	}

	white_pieces = wPawns | wKnights | wBishops | wRooks | wQueens | wKing;
	black_pieces = bPawns | bKnights | bBishops | bRooks | bQueens | bKing;

	all = white_pieces | black_pieces;

	bishops_queens = wBishops | wQueens | bBishops | bQueens;
	rooks_queens = wRooks | wQueens | bRooks | bQueens;

	if ((!wKing) | (!bKing))
		cout << "Error in position : king not found" << endl;

	white_king_sq = FirstOne(wKing);
	black_king_sq = FirstOne(bKing);
   
	rotated_90  = InitBitboard(all, index90);
	diag_a1		= InitBitboard(all, index_a1);
	diag_h1		= InitBitboard(all, index_h1);
    
	//PrintBitboard(wRooks);
	//cout << (int)(IsSet(wRooks, A1)) << endl;

	// fix castle flags if they're not right
	if (board.castle_flags & WHITE_SCASTLE){
		if (board.sq[E1] != KING || board.sq[H1] != ROOK)
			board.castle_flags &= ~WHITE_SCASTLE;
	}
	if (board.castle_flags & WHITE_LCASTLE){
		if (board.sq[E1] != KING || board.sq[A1] != ROOK)
			board.castle_flags &= ~WHITE_LCASTLE;
	}
	if (board.castle_flags & BLACK_SCASTLE){
		if (board.sq[E8] != -KING || board.sq[H8] != -ROOK)
			board.castle_flags &= ~BLACK_SCASTLE;
	}
	if (board.castle_flags & BLACK_LCASTLE){
		if (board.sq[E8] != -KING || board.sq[A8] != -ROOK)
			board.castle_flags &= ~BLACK_LCASTLE;
	}



	ComputeMaterial();

	BuildPiecetable();
	EvalPiecetable();

	InitPieceCounters();

	#if VERIFY_BOARD
		Verify("Board::Init");
	#endif

	StoreHashcode(side_to_move);
	StorePawnHashcode(side_to_move);
}


void Board::Print(){
	BITBOARD mask;
	int i;
	char c;

	#define check(bitboard, simbol) if (bitboard & mask) c = simbol;

	cout << "+---+---+---+---+---+---+---+---+" << endl << "|";

	for (i = 0; i < 64; i++){
		
		mask = OneShl(index[i]);
		c = ' ';
		check(wPawns, 'P')
		check(wRooks, 'R')
		check(wKnights, 'N')
		check(wBishops, 'B')
		check(wQueens, 'Q')
		check(wKing, 'K')
		check(bPawns, 'p')
		check(bRooks, 'r')
		check(bKnights, 'n')
		check(bBishops, 'b')
		check(bQueens, 'q')
		check(bKing, 'k')

		cout << " " << c << " |";

		if ((i % 8) == 7) {
			cout << endl << "+---+---+---+---+---+---+---+---+" << endl;
			if ((i / 8) != 7)
				cout << '|';
		}
	}

	cout << endl;
}

char* Board::PrintBuf(){
	BITBOARD mask;
	int i;
	char c;

	static char buf[1024];
	static char s[5];

	#define check(bitboard, simbol) if (bitboard & mask) c = simbol;

	strcpy(buf, "+---+---+---+---+---+---+---+---+\n|");

	for (i = 0; i < 64; i++){	
		mask = OneShl(index[i]);
		c = ' ';
		check(wPawns, 'P')
		check(wRooks, 'R')
		check(wKnights, 'N')
		check(wBishops, 'B')
		check(wQueens, 'Q')
		check(wKing, 'K')
		check(bPawns, 'p')
		check(bRooks, 'r')
		check(bKnights, 'n')
		check(bBishops, 'b')
		check(bQueens, 'q')
		check(bKing, 'k')

		s[0] = ' ';
		s[1] = c;
		s[2] = ' ';
		s[3] = '|';
		s[4] = 0;
		strcat(buf, s);

		if ((i % 8) == 7) {
			if ((i / 8) != 7)
				strcat(buf, "\n+---+---+---+---+---+---+---+---+\n|");
			else
				strcat(buf, "\n+---+---+---+---+---+---+---+---+\n");			
		}
	}

	s[0] = '\n';
	s[1] = 0;
	strcat(buf, s);

	return buf;
}

char* Board::PrintEPD(){
	static char str[256];
	
	char* p_str = str;

	int counter,i,j;

	for (i = 7; i >= 0; i--){
		counter = 0;

		for (j = 0; j < 8; j++){
			switch(sq[i*8+j]){
				case FREE: counter++; break;
				case ROOK: if (counter) *p_str++ = '0'+counter; *p_str++ = 'R'; counter = 0; break;
				case KNIGHT: if (counter) *p_str++ = '0'+counter; *p_str++ = 'N'; counter = 0; break;
				case BISHOP: if (counter) *p_str++ = '0'+counter; *p_str++ = 'B'; counter = 0; break;
				case QUEEN: if (counter) *p_str++ = '0'+counter; *p_str++ = 'Q'; counter = 0; break;
				case KING: if (counter) *p_str++ = '0'+counter; *p_str++ = 'K'; counter = 0; break;
				case PAWN: if (counter) *p_str++ = '0'+counter; *p_str++ = 'P'; counter = 0; break;
				case -ROOK: if (counter) *p_str++ = '0'+counter; *p_str++ = 'r'; counter = 0; break;
				case -KNIGHT: if (counter) *p_str++ = '0'+counter; *p_str++ = 'n'; counter = 0; break;
				case -BISHOP: if (counter) *p_str++ = '0'+counter; *p_str++ = 'b'; counter = 0; break;
				case -QUEEN: if (counter) *p_str++ = '0'+counter; *p_str++ = 'q'; counter = 0; break;
				case -KING: if (counter) *p_str++ = '0'+counter; *p_str++ = 'k'; counter = 0; break;
				case -PAWN: if (counter) *p_str++ = '0'+counter; *p_str++ = 'p'; counter = 0; break;
				default: cout << "Print EPD : cannot identify piece on square " << i*8+j << endl;
			}
		}
		
		if (counter) *p_str++ = '0'+counter;
		if (i)
			*p_str++ = '/';
	}

	*p_str++ = ' ';
	*p_str++ = (board.side_to_move ? 'w' : 'b');

	*p_str++ = ' ';
	if (castle_flags == 0){
		*p_str++ = '-';
	} else {
		if (castle_flags & WHITE_SCASTLE) *p_str++ = 'K';
		if (castle_flags & WHITE_LCASTLE) *p_str++ = 'Q';
		if (castle_flags & BLACK_SCASTLE) *p_str++ = 'k';
		if (castle_flags & BLACK_LCASTLE) *p_str++ = 'q';
	}

	*p_str++ = ' ';
	if (board.ep_square){
		*p_str++ = File(board.ep_square)+'a';
		*p_str++ = Rank(board.ep_square)+'1';
	} else {
		*p_str++ = '-';
		*p_str++ = ' ';
	}


	*p_str++ = 0;
	
	return str;
}



extern HASHCODE RandomHash[2][7][64];	// [side][piece][square]
extern HASHCODE white_hash, black_hash;	// black to move/white to move
extern HASHCODE whiteblack_hash;		// shortcut for white_hash ^ black_hash


HASHCODE Board::ComputeHashcode(int wtm){
	HASHCODE hash = 0;
	int side,piece;
	for (int i = 0; i < 64; i++){
		if (sq[i] < 0){
			piece = -sq[i];
			side = 0;
		} else {
			piece = sq[i];
			side = 1;
		}
		if (sq[i])
			hash ^= RandomHash[side][piece][i];
	}

#if 1
	if (wtm) 
		hash ^= white_hash;
	else
		hash ^= black_hash;
#endif

	return hash;
}

void Board::StoreHashcode(int wtm){
	hashcode = ComputeHashcode(wtm);
}

extern HASHCODE pawn_hash[2][64];

void Board::UpdateHash(int move, int wtm){	// must be called _before_ making a move
	int from = From(move);
	int to = To(move);
	int piece = Piece(move);
	int capture = Captured(move);
	int promote = Promote(move);

	if (promote){
		hashcode ^= Xor(RandomHash[wtm][piece][from],RandomHash[wtm][promote][to]);
		pawn_hashcode ^= pawn_hash[wtm][from];
	} else {
		hashcode ^= Xor(RandomHash[wtm][piece][from],RandomHash[wtm][piece][to]);
		if (piece == PAWN)
			pawn_hashcode ^= Xor(pawn_hash[wtm][from], pawn_hash[wtm][to]);
	}

	hashcode ^= whiteblack_hash;
	//pawn_hashcode ^= whiteblack_hash;
	
	// en passant ?
	if (capture)
	if (capture == PAWN && piece == PAWN && sq[to] == FREE){
		if (wtm){	// remove black pawn
			hashcode ^= RandomHash[BLACK][PAWN][to-8];
			pawn_hashcode ^= pawn_hash[BLACK][to-8];
		} else {	// remove white pawn
			hashcode ^= RandomHash[WHITE][PAWN][to+8];
			pawn_hashcode ^= pawn_hash[WHITE][to+8];
		}
	} else {
		hashcode ^= RandomHash[1-wtm][capture][to];
		if (capture == PAWN)
			pawn_hashcode ^= pawn_hash[1-wtm][to];
	}

	// castle ?
	if (piece == KING){
		if (wtm){
			if (from == E1)
				if (to == G1)
					hashcode ^= Xor(RandomHash[WHITE][ROOK][H1],RandomHash[WHITE][ROOK][F1]);
				 else if (to == C1)
					hashcode ^= Xor(RandomHash[WHITE][ROOK][A1],RandomHash[WHITE][ROOK][D1]);
		} else {
			if (from == E8)
				if (to == G8)
					hashcode ^= Xor(RandomHash[BLACK][ROOK][H8],RandomHash[BLACK][ROOK][F8]);
				 else if (to == C8)
					hashcode ^= Xor(RandomHash[BLACK][ROOK][A8],RandomHash[BLACK][ROOK][D8]);
		}
	}
}



HASHCODE Board::ComputePawnHashcode(int wtm){
	HASHCODE result = 0;
	for (int square = 0; square < 64; square++)
		switch(sq[square]){
			case +PAWN : result ^= pawn_hash[1][square]; break;
			case -PAWN : result ^= pawn_hash[0][square]; break;
		}

	/*
	if (wtm)
		result ^= white_hash;
	else
		result ^= black_hash;
	*/

	return result;
}

void Board::StorePawnHashcode(int wtm){
	pawn_hashcode = ComputePawnHashcode(wtm);
}




void Board::Reset(){
	board.Init("rnbqkbnr/pppppppp/8/8/8/8/PPPPPPPP/RNBQKBNR w");
	board.castle_flags = WHITE_LCASTLE | WHITE_SCASTLE | BLACK_LCASTLE | BLACK_SCASTLE;
}


int* Board::GenerateAllMoves(int* move, int wtm){
	int* last = GenerateCaptures(move, wtm);
	last = GenerateNonCaptures(last, wtm);
	return last;
}



// see if a move is valid, doesn't check if the move leaves the king in check
// returns 0 if not a good move or 1 otherwise
int Board::IsValidMove(int move, int wtm){
	int from = From(move);
	int to = To(move);
	int promote = Promote(move);
	int capture;
	int piece;
	
	int dif;


	if (move == 0) return 0;

	if (wtm){
		piece = sq[from];
		capture = -sq[to];
	} else {
		piece = -sq[from];
		capture = sq[to];
	} 
 
	if (piece != Piece(move)) return 0;
	if (piece <= 0) return 0;
	if (capture < 0) return 0;

	if (capture != Captured(move)){
		if (!(capture == 0 && to == ep_square && piece == PAWN && Captured(move) == PAWN))
			return 0;
	}

	register BITBOARD from_mask = set_mask[from];
	register BITBOARD mask = set_mask[to];

	if (wtm){	
		switch(piece){
		case KNIGHT:
			if (And(wKnights, from_mask) == 0) return 0;
			if (And(knight_attacks[from], mask)==0) return 0;
			break;
		case BISHOP:
			if (And(wBishops, from_mask) == 0) return 0;
			if (And(AttacksBishop(from), mask)==0) return 0;
			break;
		case ROOK:
			if (And(wRooks, from_mask) == 0) return 0;
			if (And(AttacksRook(from), mask)==0) return 0;
			break;
		case QUEEN:
			if (And(wQueens, from_mask) == 0) return 0;
			if (And(AttacksRook(from), mask) == 0)
			if (And(AttacksBishop(from), mask)==0) return 0;
			break;
		case PAWN:
			// check if ep
			dif = to-from;

			if (to >= A8)
				if (promote == 0) return 0;

			if (And(wPawns, from_mask)== 0) return 0;
			
			if (dif == 8) return (capture ? 0 : 1);
			else if (dif == 16) return (sq[from+8] == 0 && !capture);
			else if (dif == 7 || dif == 9)	return (capture != 0 || to == ep_square);
			return 0;
			break;
		case KING:
			if (white_king_sq != from) return 0;

			// check for castle
			if (from == E1){
				if (to == G1){
					if (Not(And(white_scastle_mask, all)))
					if ((castle_flags & WHITE_SCASTLE) && !Attacked(E1, 0) && !Attacked(F1, 0) && !Attacked(G1, 0))
						return 1;
					return 0;
				} else if (to == C1){
					if (Not(And(white_lcastle_mask, all)))
					if ((castle_flags & WHITE_LCASTLE) && !Attacked(C1, 0) && !Attacked(D1, 0) && !Attacked(E1, 0))
						return 1;
					return 0;
				}
			}
			if (And(king_attacks[from], mask) == 0) return 0;
			break;

		default:
			return 0;
		}

		return 1;

	} else {
		switch(piece){
		case KNIGHT:
			if (And(bKnights, from_mask) == 0) return 0;
			if (And(knight_attacks[from], mask)==0) return 0;
			break;
		case BISHOP:
			if (And(bBishops, from_mask) == 0) return 0;
			if (And(AttacksBishop(from), mask)==0) return 0;
			break;
		case ROOK:
			if (And(bRooks, from_mask) == 0) return 0;
			if (And(AttacksRook(from), mask)==0) return 0;
			break;
		case QUEEN:
			if (And(bQueens, from_mask) == 0) return 0;
			if (And(AttacksRook(from), mask) == 0)
			if (And(AttacksBishop(from), mask)==0) return 0;
			break;
		case PAWN:
			// check if ep
			dif = from-to;
			
			if (to <= H1)
				if (promote == 0) return 0;

			if (And(bPawns, from_mask)== 0) return 0;

			if (dif == 8) return (capture ? 0 : 1);
			else if (dif == 16) return (sq[from-8] == 0 && !capture);
			else if (dif == 7 || dif == 9) return (capture != 0 || to == ep_square);
			return 0;
			break;
		case KING:
			if (black_king_sq != from) return 0;

			// check for castle
			if (from == E8){
				if (to == G8){
					if (Not(And(black_scastle_mask, all)))
					if ((castle_flags & BLACK_SCASTLE) && !Attacked(E8, 0) && !Attacked(F8, 0) && !Attacked(G8, 0))
						return 1;
					return 0;
				} else if (to == C8){
					if (Not(And(black_lcastle_mask, all)))
					if ((castle_flags & BLACK_LCASTLE) && !Attacked(C8, 0) && !Attacked(D8, 0) && !Attacked(E8, 0))
						return 1;
					return 0;
				}
			}

			if (And(king_attacks[from], mask) == 0) return 0;

			break;

		default:
			return 0;
		}

		return 1;
	}
}





extern const int piece_v[7];

void Board::ComputeMaterial(){
	white_material =	BitCount(wRooks) + 
						BitCount(wKnights)+ 
						BitCount(wBishops)+ 
						BitCount(wQueens);
	white_pawns = BitCount(wPawns);
	black_material =	BitCount(bRooks)+ 
						BitCount(bKnights)+ 
						BitCount(bBishops)+ 
						BitCount(bQueens);
	black_pawns = BitCount(bPawns);
	material =   white_pawns * piece_v[PAWN] +
				 BitCount(wRooks) * piece_v[ROOK] + 
				 BitCount(wKnights) * piece_v[KNIGHT] + 
				 BitCount(wBishops) * piece_v[BISHOP] + 
				 BitCount(wQueens) * piece_v[QUEEN]+
				-black_pawns * piece_v[PAWN] +
				-BitCount(bRooks) * piece_v[ROOK] + 
				-BitCount(bKnights) * piece_v[KNIGHT] + 
				-BitCount(bBishops) * piece_v[BISHOP] + 
				-BitCount(bQueens) * piece_v[QUEEN];
}

void Board::VerifyMaterial(){
	Assert(white_material ==	BitCount(wRooks)+ 
								BitCount(wKnights)+ 
								BitCount(wBishops)+ 
								BitCount(wQueens));
	Assert(white_pawns == BitCount(wPawns));
	Assert(black_material ==	BitCount(bRooks) + 
								BitCount(bKnights) + 
								BitCount(bBishops)+ 
								BitCount(bQueens));
	Assert(black_pawns == BitCount(bPawns));
	Assert(material ==	 white_pawns * piece_v[PAWN] +
						 BitCount(wRooks) * piece_v[ROOK] + 
						 BitCount(wKnights) * piece_v[KNIGHT] + 
						 BitCount(wBishops) * piece_v[BISHOP] + 
						 BitCount(wQueens) * piece_v[QUEEN]+
						-black_pawns * piece_v[PAWN] +
						-BitCount(bRooks) * piece_v[ROOK] + 
						-BitCount(bKnights) * piece_v[KNIGHT] + 
						-BitCount(bBishops) * piece_v[BISHOP] + 
						-BitCount(bQueens) * piece_v[QUEEN]);
}

#include "prophet.h"

void Board::EvalPiecetable(){
	piecetable_score = EvaluateStaticPosition();
}


int* Board::GenerateAllValidMoves(int* move, int wtm){
	static int temp_list[256];
	int *m;

	SaveBoardInfo(100);

	int* last = GenerateAllMoves(temp_list, wtm);

	/*
	board.Print();
	cout << "Root moves are :" << endl;
	for (m = temp_list; m < last; m++){
		PrintMove(*m);
	}
	cout <<endl;

	PrintBitboard(board.wPawns);
	PrintBitboard(board.bPawns);
	*/

	for (m = temp_list; m < last; m++){
		board.Make(*m, 100, wtm);
		if (!Check(wtm)){
			*move++ = *m;
		}
		board.Unmake(*m, 100, wtm);
	}

	return move;
}	



void Board::InitPieceCounters(){
	piece_counter[WHITE][PAWN] = BitCount(board.wPawns);
	piece_counter[WHITE][KNIGHT] = BitCount(board.wKnights);
	piece_counter[WHITE][BISHOP] = BitCount(board.wBishops);
	piece_counter[WHITE][ROOK] = BitCount(board.wRooks);
	piece_counter[WHITE][QUEEN] = BitCount(board.wQueens);
	piece_counter[BLACK][PAWN] = BitCount(board.bPawns);
	piece_counter[BLACK][KNIGHT] = BitCount(board.bKnights);
	piece_counter[BLACK][BISHOP] = BitCount(board.bBishops);
	piece_counter[BLACK][ROOK] = BitCount(board.bRooks);
	piece_counter[BLACK][QUEEN] = BitCount(board.bQueens);
}


void Board::VerifyPieceCounter(){
	if (piece_counter[WHITE][PAWN] != BitCount(board.wPawns)){
		cout << "Bingo !" << endl;
	}
	Assert(piece_counter[WHITE][PAWN] == BitCount(board.wPawns))
	Assert(piece_counter[WHITE][KNIGHT] == BitCount(board.wKnights))
	Assert(piece_counter[WHITE][BISHOP] == BitCount(board.wBishops))
	Assert(piece_counter[WHITE][ROOK] == BitCount(board.wRooks))
	Assert(piece_counter[WHITE][QUEEN] == BitCount(board.wQueens))
	Assert(piece_counter[BLACK][PAWN] == BitCount(board.bPawns))
	Assert(piece_counter[BLACK][KNIGHT] == BitCount(board.bKnights))
	Assert(piece_counter[BLACK][BISHOP] == BitCount(board.bBishops))
	Assert(piece_counter[BLACK][ROOK] == BitCount(board.bRooks))
	Assert(piece_counter[BLACK][QUEEN] == BitCount(board.bQueens))
}


void Board::Mirror(){
	int tmp, r,f;

	MirrorAndSwap(wPawns, bPawns);
	MirrorAndSwap(wRooks, bRooks);
	MirrorAndSwap(wKnights, bKnights);
	MirrorAndSwap(wBishops, bBishops);
	MirrorAndSwap(wQueens, bQueens);
	MirrorAndSwap(wKing, bKing);
	MirrorAndSwap(white_pieces, black_pieces);
	all = ::Mirror(all);

	white_king_sq = FirstOne(wKing);
	black_king_sq = FirstOne(bKing);

	side_to_move = 1-side_to_move;

	ep_square = (7-Rank(ep_square))*8 + File(ep_square);

	castle_flags = ((castle_flags >> 2) & 3)	| ((castle_flags & 3) << 2);

	for (r = RANK1; r <= RANK4; r++)
		for (f = FILEA; f <= FILEH; f++){
			tmp = sq[(7-r)*8+f];
			sq[(7-r)*8+f] = -sq[r*8+f];
			sq[r*8+f] = -tmp;
		}

	rotated_90  = InitBitboard(all, index90);
	diag_a1		= InitBitboard(all, index_a1);
	diag_h1		= InitBitboard(all, index_h1);

	bishops_queens = ::Mirror(bishops_queens);
	rooks_queens = ::Mirror(rooks_queens);
    
	ComputeMaterial();

	BuildPiecetable();
	EvalPiecetable();

	InitPieceCounters();

	#if VERIFY_BOARD
		Verify("Board::Init");
	#endif

	StoreHashcode(side_to_move);
	StorePawnHashcode(side_to_move);


	/*
	int square[64];
	char str[256];

	for (int r = RANK1; r <= RANK8; r++)
		for (int f = FILEA; f <= FILEH; f++)
			square[(7-r)*8 + f] = -sq[r*8+f];

	char* p_str = str;

	int counter,i,j;

	for (i = 7; i >= 0; i--){
		counter = 0;

		for (j = 0; j < 8; j++){
			switch(square[i*8+j]){
				case FREE: counter++; break;
				case ROOK: if (counter) *p_str++ = '0'+counter; *p_str++ = 'R'; counter = 0; break;
				case KNIGHT: if (counter) *p_str++ = '0'+counter; *p_str++ = 'N'; counter = 0; break;
				case BISHOP: if (counter) *p_str++ = '0'+counter; *p_str++ = 'B'; counter = 0; break;
				case QUEEN: if (counter) *p_str++ = '0'+counter; *p_str++ = 'Q'; counter = 0; break;
				case KING: if (counter) *p_str++ = '0'+counter; *p_str++ = 'K'; counter = 0; break;
				case PAWN: if (counter) *p_str++ = '0'+counter; *p_str++ = 'P'; counter = 0; break;
				case -ROOK: if (counter) *p_str++ = '0'+counter; *p_str++ = 'r'; counter = 0; break;
				case -KNIGHT: if (counter) *p_str++ = '0'+counter; *p_str++ = 'n'; counter = 0; break;
				case -BISHOP: if (counter) *p_str++ = '0'+counter; *p_str++ = 'b'; counter = 0; break;
				case -QUEEN: if (counter) *p_str++ = '0'+counter; *p_str++ = 'q'; counter = 0; break;
				case -KING: if (counter) *p_str++ = '0'+counter; *p_str++ = 'k'; counter = 0; break;
				case -PAWN: if (counter) *p_str++ = '0'+counter; *p_str++ = 'p'; counter = 0; break;
				default: cout << "Print EPD : cannot identify piece on square " << i*8+j << endl;
			}
		}
		
		if (counter) *p_str++ = '0'+counter;
		if (i)
			*p_str++ = '/';
	}

	*p_str++ = ' ';
	*p_str++ = (board.side_to_move ? 'b' : 'w');

	*p_str++ = ' ';
	if (castle_flags == 0){
		*p_str++ = '-';
	} else {
		if (castle_flags & BLACK_SCASTLE) *p_str++ = 'K';
		if (castle_flags & BLACK_LCASTLE) *p_str++ = 'Q';
		if (castle_flags & WHITE_SCASTLE) *p_str++ = 'k';
		if (castle_flags & WHITE_LCASTLE) *p_str++ = 'q';
	}

	*p_str++ = ' ';
	if (board.ep_square){
		*p_str++ = File(board.ep_square)+'a';
		*p_str++ = (7-Rank(board.ep_square))+'1';
	} else {
		*p_str++ = '-';
		*p_str++ = ' ';
	}


	*p_str++ = 0;

	//cout << "[" << str << "]" << endl;

	Init(str);
	*/
}
