#include "prophet.h"
#include "board.h"
#include <string.h>
#include "movelist.h"
#include <iostream>

using namespace std;


int piecetable[2][7][64];


static int pawn_pcsq[64] = {
	   0,    0,    0,    0,    0,    0,    0,    0,
	   0,    0,    0, -400, -400,    0,    0,    0,
	  10,   20,   30,  -80,  -80,   30,   20,   10,
	  20,   45,   60,   80,   80,   60,   45,   20,
	  10,   35,   40,   60,   60,   40,   35,   10,
	  40,   80,  120,  160,  160,  120,   80,   40,
	  50,  100,  150,  200,  200,  150,  100,   50,
	   0,    0,    0,    0,    0,    0,    0,    0,
};

static int knight_pcsq[64] = {
	-10, -30, -10, -10, -10, -10, -30, -10,
	-10,   0,   0,   0,   0,   0,   0, -10,
	-10,   0,   5,   5,   5,   5,   0, -10,
	 -0,   0,   5,  10,  10,   5,	0, -0,
	 -0,   0,   5,  10,  10,   5,	0, -0,
	-10,   0,   5,   5,   5,   5,   0, -10,
	-10,   0,   0,   0,   0,   0,   0, -10,
	-10, -10, -10, -10, -10, -10, -10, -10,
};

static int bishop_pcsq[64] = {
	-10, -10, -20, -10, -10, -20, -10, -10,
	-10,   0,   0,   0,   0,   0,   0, -10,
	-10,   0,   5,   5,   5,   5,   0, -10,
	-10,   0,   5,  10,  10,   5,   0, -10,
	-10,   0,   5,  10,  10,   5,   0, -10,
	-10,   0,   5,   5,   5,   5,   0, -10,
	-10,   0,   0,   0,   0,   0,   0, -10,
	-10, -10, -10, -10, -10, -10, -10, -10,
};


static int king_pcsq[64] = {
	  9,   9,   8,  -4,   0,  -4,   8,   3,
	  8,   7,  -4,  -4,  -4,  -4,   3,   0,
	 -8,  -8,  -8,  -8,  -8,  -8,  -8,  -8,
	 -8,  -8,  -8,  -8,  -8,  -8,  -8,  -8,
	 -8,  -8,  -8,  -8,  -8,  -8,  -8,  -8,
	 -8,  -8,  -8,  -8,  -8,  -8,  -8,  -8,
	 -8,  -8,  -8,  -8,  -8,  -8,  -8,  -8,
	 -8,  -8,  -8,  -8,  -8,  -8,  -8,  -8,
};

static int king_endgame_pcsq[64] = {
	  0,  10,  20,  30,  30,  20,  10,   0,
	 10,  20,  30,  40,  40,  30,  20,  10,
	 20,  30,  40,  50,  50,  40,  30,  20,
	 30,  40,  50,  60,  60,  50,  40,  30,
	 30,  40,  50,  60,  60,  50,  40,  30,
	 20,  30,  40,  50,  50,  40,  30,  20,
	 10,  20,  30,  40,  40,  30,  20,  10,
	  0,  10,  20,  30,  30,  20,  10,   0
};


static int flip[64] = {
	A8, B8, C8, D8, E8, F8, G8, H8,
	A7, B7, C7, D7, E7, F7, G7, H7,
	A6, B6, C6, D6, E6, F6, G6, H6,
	A5, B5, C5, D5, E5, F5, G5, H5,
	A4, B4, C4, D4, E4, F4, G4, H4,
	A3, B3, C3, D3, E3, F3, G3, H3,
	A2, B2, C2, D2, E2, F2, G2, H2,
	A1, B1, C1, D1, E1, F1, G1, H1
};


void InitTable(int* table, const int* values){
	for (int i = 0; i < 64; i++)
		*table++ = *values++;
}

void AddTable(int* table, const int* values, int multiplier = 1){
	for (int i = 0; i < 64; i++, values++){
		//cout << "table[" << i << "] = " << (*values) << endl;
		*table++ += (*values) * multiplier;
	}
}

void AddTableReversed(int* table, const int* values, int multiplier = 1){
	for (int i = 0; i < 64; i++)
		*table++ = values[flip[i]] * multiplier;
}

void AddValueToTable(int* table, int value){
	for (int i = 0; i < 64; i++, table++)
		*table += value;
}


#include <fstream>
#include <stdio.h>

using namespace std;

extern unsigned char index[];


void PrintScore(int* array){

	char buf[32];
	
	for (int i = 0; i < 64; i++){
		int v = array[index[i]];
		sprintf(buf, "%3d ", v);
		cout << buf;
		if ((i & 7) == 7) cout << endl;
	}
}

void PrintTable(char* msg, int* tbl, ostream& fp = cout){
	fp << msg << endl;

	for (int i = 0; i < 64; i++){
		fp.width(4);
		fp << tbl[flip[i]] << " ";
		if ((i & 7) == 7)
			fp << endl;
	}
	fp << endl;
}

void PrintTables(ostream& fp = cout){
	PrintTable("White King",	piecetable[WHITE][KING], fp);
	PrintTable("White Pawns",	piecetable[WHITE][PAWN], fp);
	PrintTable("White Knights", piecetable[WHITE][KNIGHT], fp);
	PrintTable("White Bishops", piecetable[WHITE][BISHOP], fp);
	PrintTable("White Rooks",	piecetable[WHITE][ROOK], fp);
	PrintTable("White Queen",	piecetable[WHITE][QUEEN], fp);
	PrintTable("Black King",	piecetable[BLACK][KING], fp);
	PrintTable("Black Pawns",	piecetable[BLACK][PAWN], fp);
	PrintTable("Black Knights", piecetable[BLACK][KNIGHT], fp);
	PrintTable("Black Bishops", piecetable[BLACK][BISHOP], fp);
	PrintTable("Black Rooks",	piecetable[BLACK][ROOK], fp);
	PrintTable("Black Queen",	piecetable[BLACK][QUEEN], fp);
}


int castle_penalty[2];
int undeveloped_pieces_penalty[2];

const int ROOK_ON_8TH_BONUS = 50;

void AddRookOn8ThBonus(){
	int i;
	for (i = 0; i < 8; i++){
		piecetable[WHITE][ROOK][A8+i] = ROOK_ON_8TH_BONUS;
		piecetable[BLACK][ROOK][A1+i] = ROOK_ON_8TH_BONUS;
	}
}

void BuildPiecetable(){	// construct the piecetable based on game phase and position
	memset(piecetable, 0, sizeof(piecetable));

	int w_material = BitCount(board.wRooks) * piece_v[ROOK] + 
				 BitCount(board.wKnights) * piece_v[KNIGHT] + 
				 BitCount(board.wBishops) * piece_v[BISHOP] + 
				 BitCount(board.wQueens) * piece_v[QUEEN];
	int b_material = BitCount(board.bRooks) * piece_v[ROOK] + 
				 BitCount(board.bKnights) * piece_v[KNIGHT] + 
				 BitCount(board.bBishops) * piece_v[BISHOP] + 
				 BitCount(board.bQueens) * piece_v[QUEEN];


	if (MoveNumber() < 30){
		if (board.castle_flags & (WHITE_SCASTLE | WHITE_LCASTLE))
			castle_penalty[WHITE] = -300;
		else 
			castle_penalty[WHITE] = 0;

		if (board.castle_flags & (BLACK_SCASTLE | BLACK_LCASTLE))
			castle_penalty[BLACK] = -300;
		else 
			castle_penalty[BLACK] = 0;

		undeveloped_pieces_penalty[WHITE] = undeveloped_pieces_penalty[BLACK] = -250;

	} else {
		castle_penalty[WHITE] = castle_penalty[BLACK] = 0;
		undeveloped_pieces_penalty[WHITE] = undeveloped_pieces_penalty[BLACK] = 0;
	}

	
	AddTable(piecetable[WHITE][PAWN], pawn_pcsq, 1);
	AddTable(piecetable[WHITE][KNIGHT], knight_pcsq, 10);
	AddTable(piecetable[WHITE][BISHOP], bishop_pcsq, 10);
	AddTableReversed(piecetable[BLACK][PAWN], pawn_pcsq, 1);
	AddTableReversed(piecetable[BLACK][KNIGHT], knight_pcsq, 10);
	AddTableReversed(piecetable[BLACK][BISHOP], bishop_pcsq, 10);

	
	if (b_material >= 9*PAWN_V) 
		AddTable(piecetable[WHITE][KING], king_pcsq, 10);
	else
		AddTable(piecetable[WHITE][KING], king_endgame_pcsq, 10);

	if (w_material >= 9*PAWN_V) 
		AddTableReversed(piecetable[BLACK][KING], king_pcsq, 10);
	else
		AddTableReversed(piecetable[BLACK][KING], king_endgame_pcsq, 10);
	
	AddRookOn8ThBonus();
	

	//PrintTables();
}


int EvaluateStaticPosition(){
	int i;
	int result = 0;

	//for (i = 0; i < 64; i++) cout << board.sq[i] << " ";
	//cout << endl;

	int* sq = board.sq;
	for (i = 0; i < 64; i++, sq++){
		//cout << *sq << " ";
		if (*sq > 0)
			result += piecetable[WHITE][*sq][i];
		else if (*sq < 0)
			result -= piecetable[BLACK][-(*sq)][i];			
	}
	//cout << endl;

	//cout << "->[" << result << "]<-" << endl;

	return result;
}
