#include "search.h"
#include "board.h"
#include "move.h"
#include "hash.h"
#include <iostream>
#include "input.h"
#include "stats.h"
#include "tree.h"
#include "history.h"
#include "rootmove.h"
#include "swap.h"
#include "utils.h"
#include "hashlist.h"

using namespace std;


extern int move_list[];
extern int last_move[];


static int min_depth = 0;

void SortMoves(int* first, int* last, int* p_score);

void FindFirstMove(int *first,int *last, int*score);

#define HASH_QNODES 0

extern int score[];


int FindBestMove(int* m, int* last, int *p, int wtm){
	int i = 0;
	int j = 0;
	int max = *p;
	
	for (; m < last;){
		if (*p > max){
			max = *p;
			i = j;
		}
		m++;
		p++;
		j++;
	}
	return i;
}


int QSearch(int alpha, int beta, int ply, int depth, int wtm){
	int x,i,eval;
	int *p, *m;

	stats.qnodes++;

	CheckSearchTime();

	// draw by repetition check
	tree[ply].same_position_or_rule50 = 0;
	if (HashListRepetition2x()){
		tree[ply].same_position_or_rule50 = 1;
		return DrawScore(wtm);
	}

	int *m_list = (ply == 0) ? move_list : FirstMove(ply);

	if (Check(wtm)){
		pvlen[ply] = 0;	
		SaveBoardInfo(ply);

		int *last = board.GenerateCheckEvasions(m_list, wtm);
		last_move[ply] = last-move_list;
		if (last==m_list) return -MATE+ply*10;

		pvlen[ply] = 0;	

		int gamma = -INF;
		
		for (m = m_list; m < last; m++){
			i = 0;

			board.Make(m[i], ply, wtm);

			x = -QSearch(-beta, -Max(gamma, alpha), ply+1, depth-PLY_DEPTH, 1-wtm);

			board.Unmake(m[i], ply, wtm);

			if (input_flags.time_left <= 0) return 0;		
			
			if (x > gamma){
				if (x >= beta) {
					UpdatePV(ply, m[i])
					return beta;
				}
				gamma = x;
			}
		}

		return gamma;
	}


	eval = x = board.Evaluate(alpha, beta, ply, wtm);

	if (x > alpha){
		if (x >= beta){
			return beta;
		}
		alpha = x;
	} 

	int alpha0 = alpha;

	pvlen[ply] = 0;	
	SaveBoardInfo(ply);

	int *last = board.GenerateCaptures(m_list, wtm);
	last_move[ply] = last-move_list;


	int delta = alpha-800-(wtm ? board.material : -board.material);

	int* p_score = score + (m_list - move_list);

	int* last_move = m_list;
	int* last_score = p_score;

	for (p = p_score, m = m_list; m < last; m++, p++){

		if (Captured(*m) == KING) return beta;

		int c = Captured(*m);
		int p = Piece(*m);

		if (piece_v[c] + 800 + eval >= alpha){
			if (piece_v[p] < piece_v[c]){
				*last_score++ = piece_v[c] - p;
				*last_move++ = *m;
			} else {
				x = Swap(From(*m), To(*m), wtm);
				if (x > 0){
					*last_score++ = x - p;
					*last_move++ = *m;
				}
			}
		}	
	}

	last = last_move;



	/* loop through the moves */
	for (m = m_list; m < last; m++, p_score++){
		i = FindBestMove(m, last, p_score, wtm);

		if (p_score[i] + eval + 800 < alpha) break;

		board.Make(m[i], ply, wtm);

		int legal = !Check(wtm);
		if (legal)
			x = -QSearch(-beta, -alpha, ply+1, depth-PLY_DEPTH, 1-wtm);

		board.Unmake(m[i], ply, wtm);

		if (input_flags.time_left <= 0) return 0;

		if (legal && x > alpha){
			if (x >= beta){
				return beta;
			}
			alpha = x;

			UpdatePV(ply, m[i]);
		}

		if (i > 0){
			m[i] = m[0];
			p_score[i] = p_score[0];
		}
	}

	return alpha;
}

void SortMoves(int* first, int* last, int* p_score){
	int tmp;
	int* best;
	int* best_v;

	best = first;
	best_v = p_score;

	int* p = p_score+1;
	int* m = first+1;
	for (; m < last; m++, p++){
		tmp = *p;
		if (tmp > *best_v){
			best_v = p;
			best = m;
		}
	}

	if (best != first){
		tmp = *first; *first = *best;*best = tmp;
		tmp = *p_score; *p_score = *best_v; *best_v = tmp;
	}

}
