#ifndef _C_VERIFY_
#define _C_VERIFY_
#include "board.h"
#include "move.h"
#include <iostream>

using namespace std;

extern int current_move[];
extern int current_ply;


void Board::PrintDebug(){
	cout << "Move list :";
	for (int i = 0; i <= current_ply; i++)
		PrintMove(current_move[i]);
	cout << endl;

	Print();

	/*
	PrintBitboard(white_pieces); 
	PrintBitboard(wPawns | wKnights | wBishops | wRooks | wQueens | wKing);
	PrintBitboard(wPawns);
	PrintBitboard(wKnights);
	PrintBitboard(wBishops);
	PrintBitboard(wRooks);
	PrintBitboard(wQueens);
	PrintBitboard(wKing);
	*/

	//PrintBitboard(black_pieces);
	//PrintBitboard(bPawns);
	//PrintBitboard(bQueens);

	/*
	PrintBitboard(bishops_queens);
	PrintBitboard(wBishops);
	PrintBitboard(wQueens);
	PrintBitboard(bBishops);
	PrintBitboard(bQueens);
	*/
}

#undef Assert
#define Assert(cond) if (!(cond)) { cout << "Verify error in " << where << endl; PrintDebug(); error(#cond, __FILE__, __LINE__);	}


void Board::VerifyHash(int wtm){
	char* where = "verify_hash";
	Assert(hashcode == ComputeHashcode(wtm));
}

void Board::VerifyPawnHash(int wtm){
	char* where = "verify_pawn_hash";
	Assert(pawn_hashcode == this->ComputePawnHashcode(wtm));
}


void Board::Verify(char* where){
	int i;

	Assert(white_pieces == (wPawns | wKnights | wBishops | wRooks | wQueens | wKing));
	Assert(black_pieces == (bPawns | bKnights | bBishops | bRooks | bQueens | bKing));
	Assert((white_pieces & black_pieces) == 0);

	Assert(BitCount(wKing) == 1);
	Assert(BitCount(bKing) == 1);
	Assert(white_king_sq == FirstOne(wKing));
	Assert(black_king_sq == FirstOne(bKing));

	Assert(bishops_queens == (wBishops | wQueens | bBishops | bQueens));
	Assert(rooks_queens == (wRooks | wQueens | bRooks | bQueens));

	// verify rotated bitboards
	Assert(all == (white_pieces | black_pieces));

	for (i = 0; i < 64; i++){	
		int bit = ((all & OneShl(i)) != 0);
		int bit90 = ((rotated_90 & set_mask_90[i]) != 0);
		int bitA1 = ((diag_a1 & set_mask_a1[i]) != 0);
		int bitH1 = ((diag_h1 & set_mask_h1[i]) != 0);

		Assert(bit == bit90 && bit == bitA1 && bit == bitH1);
	}

	// verify sq using pieces
	for (i = 0; i < 64; i++){
		BITBOARD mask = OneShl(i);
		switch(sq[i]){
		case FREE :
			Assert((mask & all) == 0);
			break;
		case KNIGHT :
			Assert(mask & wKnights);
			break;
		case BISHOP:
			//PrintBitboard(mask);
			//PrintBitboard(wBishops);
			//cout << i << endl;
			Assert(mask & wBishops);
			Assert(mask & bishops_queens);
			break;
		case ROOK:
			Assert(mask & wRooks);
			Assert(mask & rooks_queens);
			break;
		case QUEEN:
			Assert(mask & wQueens);
			Assert(mask & rooks_queens);
			Assert(mask & bishops_queens);
			break;
		case KING:
			Assert(mask & wKing);
			break;
		case PAWN:
			Assert(mask & wPawns);
			break;
		case -KNIGHT :
			Assert(mask & bKnights);
			break;
		case -BISHOP:
			Assert(mask & bBishops);
			Assert(mask & bishops_queens);
			break;
		case -ROOK:
			Assert(mask & bRooks);
			Assert(mask & rooks_queens);
			break;
		case -QUEEN:
			Assert(mask & bQueens);
			Assert(mask & rooks_queens);
			Assert(mask & bishops_queens);
			break;
		case -KING:
			Assert(mask & bKing);
			break;
		case -PAWN:
			Assert(mask & bPawns);
			break;
		default:
			Assert(false);
		}
	}

	// verify castle flags 
	if (castle_flags & WHITE_SCASTLE)
		Assert(sq[E1] == KING && sq[H1] == ROOK);
	if (castle_flags & WHITE_LCASTLE)
		Assert(sq[E1] == KING && sq[A1] == ROOK);
	if (castle_flags & BLACK_SCASTLE)
		Assert(sq[E8] ==-KING && sq[H8] ==-ROOK);
	if (castle_flags & BLACK_LCASTLE)
		Assert(sq[E8] ==-KING && sq[A8] ==-ROOK);
}

#endif
